#! /usr/bin/env python
# -*- coding: iso-8859-1 -*-

# This file is part of the SMUML toolset.
# Copyright (C) 2007 Helsinki University of Technology.
#
# The SMUML toolset is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License version 2,
# as published by the Free Software Foundation.
#
# The SMUML toolset is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with the SMUML toolset.  If not, see <http://www.gnu.org/licenses/>.

# coralutils.py
#
# 2006-10-30  Jori Dubrovin:  Initial version
#

"""Helper functions related to Coral and UML."""

import coral


uml = coral.lang.UML14

def umlSort(elements):
    """Return a sorted list of UML elements.

    This function is used to sort the UML elements at an association
    end with multiplicity greater than one.  This way, the same model
    always produces the elements in the same order, regardless of the
    order in which Coral returns them.

    Param elements: an iterable over Coral UML elements.

    Return value: a sorted list containing the same elements.
    """
    def compareXmiIds(elem1, elem2):
        """Compare the XMI ids of two Coral UML elements."""
        return cmp(str(elem1.__XMIid__()), str(elem2.__XMIid__()))
    elems = list(elements)
    elems.sort(compareXmiIds)
    return elems
