#! /usr/bin/env python
# -*- coding: iso-8859-1 -*-

# This file is part of the SMUML toolset.
# Copyright (C) 2007 Helsinki University of Technology.
#
# The SMUML toolset is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License version 2,
# as published by the Free Software Foundation.
#
# The SMUML toolset is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with the SMUML toolset.  If not, see <http://www.gnu.org/licenses/>.

# options.py
#
# 2007-05-04  Jori Dubrovin:  Added linear Enter predicate
# 2007-02-05  Jori Dubrovin:  Version 1.00
# 2006-12-04  Jori Dubrovin:  Using optparse for options
# 2006-08-31  Jori Dubrovin:  Added encodeStateExclusion
# 2006-08-16  Jori Dubrovin:  Added checks for run-time errors
# 2006-07-30  Jori Dubrovin:  Initial version
#

"""Global options for translation of UML to NuSMV."""

import optparse


opt = None

def initializeOptions(version):
    """Read command line and initialize the options structure.

    Param version: Uboco version number as a string.

    Return value: (infilename, outfilename).
    """
    p = optparse.OptionParser(usage='python %prog [options] ' +
                              'infile.xmi outfile.smv',
                              version='Uboco ' + version)
    p.add_option('--sloppy-input', action='store_true', default=False,
                 help='accept inconsistent UML models')
    p.add_option('--int-bits', type='int', default=8, metavar='N',
                 help='number of bits in the int type [default: 8]')
    p.add_option('--instances', type='int', default=1, metavar='N',
                 help='number of instances per class [default: 1]')
    p.add_option('--specific-instances', action='append', default=[],
                 metavar='CLASSNAME:N',
                 help='number of instances of a specific class')
    p.add_option('--queue-size', type='int', default=2, metavar='N',
                 help='size of event queues [default: 2]')
    p.add_option('--specific-queue-size', action='append', default=[],
                 metavar='CLASSNAME:N',
                 help='size of event queue of a specific class')
    p.add_option('--check-deadlock', action='store_true', default=False,
                 help='check deadlocks')
    p.add_option('--check-implicit-consumption', action='store_true',
                 default=False,
                 help='check implicit consumption of events')
    p.add_option('--check-assertions', action='store_true', default=False,
                 help='check Jumbala assertion errors')
    p.add_option('--check-runtime-errors', action='store_true', default=False,
                 help='check Jumbala run-time errors')
    p.add_option('--encode-interleaving', action='store_true', default=False,
                 help='encode interleaving execution semantics')
    p.add_option('--encode-static', action='store_true', default=False,
                 help='encode static step semantics [default: dynamic steps]')
    p.add_option('--old-enter', action='store_true', default=False,
                 help='encode superlinear enter predicates [default: linear]')
    p.add_option('--allow-queue-overdraft', action='store_true', default=False,
                 help='allow exceeding queue capacity in step semantics')
    # Could add: --encode-state-exclusion
    global opt
    (opt, args) = p.parse_args()
    if len(args) != 2:
        p.error('wrong number of arguments')
    if opt.int_bits < 2 or opt.int_bits > 32:
        p.error('value of --int-bits must be between 2 and 32')
    return (args[0], args[1])
