import sys
import sets

import coral

import actlang
import actlang.expr

UML14 = coral.lang.UML14


class Info:
    def __init__(self):
        self.this_read = sets.Set()
        self.this_write = sets.Set()
        self.other_read = sets.Set()
        self.other_write = sets.Set()
        self.signals_sent = sets.Set()
        self.objects_created = sets.Set()
    def __str__(self):
        s = ""
        s += "this_read = {" + self._str_set(self.this_read) + "}, "
        s += "this_write = {" + self._str_set(self.this_write) + "}, "
        s += "other_read = {" + self._str_set(self.other_read) + "}, "
        s += "other_write = {" + self._str_set(self.other_write) + "}, "
        s += "signals_sent = {" + self._str_signal_set(self.signals_sent) + "}, "
        s += "objects_created = {" + self._str_class_set(self.objects_created) + "}"
        return s
    def _str_set(self, s):
        r = ""
        sep = ""
        for (c,f) in s:
            r += sep + str(c.name)+"::"+str(f.name)
            sep = ","
        return r
    def _str_signal_set(self, s):
        r = ""
        sep = ""
        for (signal,target) in s:
            assert signal.oclIsKindOf(UML14.Signal)
            if type(target) is str:
                assert target == "this"
                r += sep + str(signal.name)+" to "+str(target)
            else:
                assert target.oclIsKindOf(UML14.Classifier)
                r += sep + str(signal.name)+" to "+str(target.name)
            sep = ","
        return r
    def _str_class_set(self, s):
        r = ""
        sep = ""
        for c in s:
            r += sep + str(c.name)
            sep = ","
        return r
    def union(self, info2):
        assert isinstance(info2, Info)
        self.this_read = self.this_read.union(info2.this_read)
        self.this_write = self.this_write.union(info2.this_write)
        self.other_read = self.other_read.union(info2.other_read)
        self.other_write = self.other_write.union(info2.other_write)
        self.signals_sent = self.signals_sent.union(info2.signals_sent)
        self.objects_created = self.objects_created.union(info2.objects_created)




class VarRef:
    def __init__(self, var_class, var_name):
        if var_class == None:
            assert var_name == None
        elif type(var_class) is str:
            assert var_class == "this"
            assert (var_name == None or
                    var_name.oclIsKindOf(UML14.Attribute) or
                    var_name.oclIsKindOf(UML14.AssociationEnd))
        else:
            assert var_class.oclIsKindOf(UML14.Classifier)
            assert var_name
            assert (var_name.oclIsKindOf(UML14.Attribute) or
                    var_name.oclIsKindOf(UML14.AssociationEnd))
        self.var_class = var_class
        self.var_name = var_name
    def is_none(self):
        return self.var_class == None and self.var_name == None
    def is_this_reference(self):
        return (type(self.var_class) is str and
                self.var_class == "this" and
                self.var_name == None)
    def is_this_dot_var(self):
        return (type(self.var_class) is str and
                self.var_class == "this" and
                self.var_name != None)
    def is_class_dot_var(self):
        return (self.var_class and
                type(self.var_class) is not str and
                self.var_class.oclIsKindOf(UML14.Classifier) and
                self.var_name != None)
    def add_read_to_info(self, info, this_class):
        assert isinstance(info, Info)
        assert this_class.oclIsKindOf(UML14.Class)
        if self.is_none():
            return
        elif self.is_this_reference():
            return
        elif self.is_this_dot_var():
            info.this_read.add((this_class, self.var_name))
        elif self.is_class_dot_var():
            info.other_read.add((self.var_class, self.var_name))
    def add_write_to_info(self, info, this_class):
        assert isinstance(info, Info)
        assert this_class.oclIsKindOf(UML14.Class)
        if self.is_none():
            return
        elif self.is_this_reference():
            return
        elif self.is_this_dot_var():
            info.this_write.add((this_class, self.var_name))
        elif self.is_class_dot_var():
            info.other_write.add((self.var_class, self.var_name))


def _find_attribute_or_association(c, name):
    assert c.oclIsKindOf(UML14.Class)

    attr = None
    for f in c.feature:
        if not f.oclIsKindOf(UML14.Attribute):
            continue
        if f.name == name:
            if attr:
                raise "Attribute '"+name+"' define twice in class '"+str(c.name)+"'"
            attr = f

    assoc_var = None
    for assoc_end in c.association:
        assert assoc_end.oclIsKindOf(UML14.AssociationEnd)
        assoc = assoc_end.association
        assert assoc.oclIsKindOf(UML14.Association)
        assert len(assoc.connection) == 2
        other_assoc_end = assoc.connection[0]
        if assoc_end == other_assoc_end:
            other_assoc_end = assoc.connection[1]
        assert other_assoc_end != assoc_end

        assoc_name = str(other_assoc_end.name)
        if assoc_name == name:
            if not other_assoc_end.isNavigable:
                print "Warning: ignoring a non-navigable association end with name '"+name+"'"
            if assoc_var:
                raise "Association '"+name+"' define twice in class '"+str(c.name)+"'"
            if attr:
                raise "'"+name+"' defined both as association and attribute in class '"+str(c.name)+"'"
            assoc_var = other_assoc_end

            if other_assoc_end.multiplicity.range:
                if other_assoc_end.multiplicity.range.lower[0] > 1 or other_assoc_end.multiplicity.range.upper[0] != 1:
                    raise SimError("Association with multiplicities other than 0..1 and 1..1 not yet supported")
                if assoc_end.aggregation != "none" or other_assoc_end.aggregation != "none":
                    raise SimError("Aggregation and composition associations not yet handled")
                if assoc_end.targetScope != "instance" or other_assoc_end.targetScope != "instance":
                    raise SimError("Association end target scopes other than 'instance' not yet handled")
    assert not (attr and assoc_var)
    if not attr and not assoc_var:
        raise "Class '"+str(c.name)+"' has no association or attribute '"+str(name)+"'"
    if attr:
        return attr
    if assoc_var:
        return assoc_var
    return None


def _get_type_class(element):
    if element.oclIsKindOf(UML14.AssociationEnd):
        assert element.participant
        assert element.participant.oclIsKindOf(UML14.Class)
        return element.participant
    if element.oclIsKindOf(UML14.Attribute):
        assert element.type
        assert element.type.oclIsKindOf(UML14.Classifier)
        return element.type
    assert(False)


def analyze_statement(model, c, stmt_str):
    #print "Analyzing statement '"+str(stmt_str)+"' in class '"+str(c.name)+"'"

    tree = actlang.grammar.parse(str(stmt_str))
    stmts = [stmt for stmt in tree.api_getTopStatements() if
             not isinstance(stmt, actlang.statement.EmptyStatement)]
    info = Info()

    for stmt in stmts:
        #print "Analyzing substatement '"+str(stmt)+"' in class '"+str(c.name)+"'"
        new_info = _analyze_statement(model, c, stmt)
        #print new_info
        info.union(new_info)
    return info

def analyze_expression(model, c, expr_str):
    stmt = "assert("+str(expr_str)+");"
    tree = actlang.grammar.parse(stmt)
    stmts = [stmt for stmt in tree.api_getTopStatements() if
             not isinstance(stmt, actlang.statement.EmptyStatement)]
    assert len(stmts) == 1
    stmt = stmts[0]
    assert isinstance(stmt, actlang.statement.AssertStatement)
    (info,t,vref) = _analyze_expr(model, c, stmt.api_getExpr())
    vref.add_read_to_info(info, c)
    if info.this_write or info.other_write or info.signals_sent or info.objects_created:
        # This is just a test...
        raise "An expression with a side effect"
    return info

    


def _find_signal(model, name):
    signals = find_model_elements(model, UML14.Signal)
    for signal in signals:
        assert signal.name
        if str(signal.name) == name:
            return signal
    return None

def _find_class(model, name):
    classes = find_model_elements(model, UML14.Class)
    name = str(name)
    for c in classes:
        assert c.name
        if str(c.name) == name:
            return c
    return None


def _analyze_statement(model, c, e):
    assert model.oclIsKindOf(UML14.Model)
    assert c.oclIsKindOf(UML14.Class)
    assert (isinstance(e, actlang.expr.Expr) or
            isinstance(e, actlang.statement.Statement))
    #print "S: "+str(e)
    #print e.__class__

    if isinstance(e, actlang.statement.ExprStatement):
        (info,t,vref) = _analyze_expr(model, c, e.api_getExpr())
        return info

    elif isinstance(e, actlang.statement.SendStatement):
        arg_list = e.api_getArgList()
        info = Info()
        for e2 in arg_list:
            (info1,t1,vref1) = _analyze_expr(model, c, e2)
            vref1.add_read_to_info(info1, c)
            info.union(info1)

        (rec_info, rec_t, rec_vref) = _analyze_expr(model, c, e.api_getTarget())
        if rec_vref.is_none():
            signal_target = rec_t
        elif rec_vref.is_this_reference():
            signal_target = rec_t
            #signal_target = "this"
        elif rec_vref.is_this_dot_var():
            signal_target = rec_t
        elif rec_vref.is_class_dot_var():
            signal_target = rec_t
        else:
            assert False
        rec_vref.add_read_to_info(rec_info, c)
        info.union(rec_info)

        signal_name = e.api_getSignalName()
        signal = _find_signal(model, signal_name)
        assert signal
        info.signals_sent.add((signal,signal_target))

        return info

    elif isinstance(e, actlang.statement.AssertStatement):
        (info,t,vref) = _analyze_expr(model, c, e.api_getExpr())
        vref.add_read_to_info(info, c)
        if info.this_write or info.other_write or info.signals_sent or info.objects_created:
            # This is just a test...
            raise "An assert statement with a side effect"
        return info

    else:
        print "Not yet handled: "+str(e)
        assert False



def _analyze_expr(model, c, e):
    assert model.oclIsKindOf(UML14.Model)
    assert c.oclIsKindOf(UML14.Class)
    assert isinstance(e, actlang.expr.Expr)
    #print "E: "+str(e)
    #print e.__class__
    
    if isinstance(e, actlang.expr.ParenExpr):
        return _analyze_expr(model, c, e.api_getInnerExpr())

    elif isinstance(e, actlang.names.SimpleName):
        names = e.api_getParts()
        assert len(names) == 1 
        name = names[0]
        element = _find_attribute_or_association(c, name)
        if not element:
            raise "Class '"+str(c.name)+"' has no attribute or navigable association called '"+str(name)+"'"
        return (Info(), _get_type_class(element), VarRef("this", element))

    elif isinstance(e, actlang.names.QualifiedName):
        path = e.api_getParts()
        (t1,vref1) = (c, VarRef("this", None))
        info = Info()
        i = 0;
        for field in path:
            if field == "this":
                if t1 == c and vref1.is_this_reference():
                    (t1,vref1) = (c,VarRef("this", None))
                else:
                    (t1,vref1) = (t1,VarRef(None, None))
                continue
            if vref1.is_none():
                assert t1.oclIsKindOf(UML14.Classifier)
                element = _find_attribute_or_association(t1, field)
                (t1,vref1) = (_get_type_class(element), VarRef(t1, element))
            elif vref1.is_this_reference():
                element = _find_attribute_or_association(c, field)
                (t1,vref1) = (_get_type_class(element),VarRef("this", element))
            elif vref1.is_this_dot_var():
                vref1.add_read_to_info(info, c)
                assert t1.oclIsKindOf(UML14.Classifier)
                element = _find_attribute_or_association(t1, field)
                (t1,vref1) = (_get_type_class(element), VarRef(t1, element))
            elif vref1.is_class_dot_var():
                vref1.add_read_to_info(info, c)
                assert t1.oclIsKindOf(UML14.Classifier)
                element = _find_attribute_or_association(t1, field)
                (t1,vref1) = (_get_type_class(element), VarRef(t1, element))
            else:
                assert False
        return (info, t1, vref1)


    elif isinstance(e, actlang.expr.PrimaryFieldAccessExpr):
        (info1,t1,vref1) = _analyze_expr(model, c, e.api_getReferenceExpr())
        #print info1, t1, vref1
        field = e.api_getFieldName()
        if vref1.is_none():
            assert t1.oclIsKindOf(UML14.Classifier)
            element = _find_attribute_or_association(t1, field)
            return (info1, _get_type_class(element), VarRef(t1, element))
        elif vref1.is_this_reference():
            element = _find_attribute_or_association(c, field)
            return (info1, _get_type_class(element), VarRef("this", element))
        elif vref1.is_this_dot_var():
            vref1.add_read_to_info(info1, c)
            assert t1.oclIsKindOf(UML14.Classifier)
            element = _find_attribute_or_association(t1, field)
            return (info1, _get_type_class(element), VarRef(t1, element))
        elif vref1.is_class_dot_var():
            vref1.add_read_to_info(info1, c)
            assert t1.oclIsKindOf(UML14.Classifier)
            element = _find_attribute_or_association(t1, field)
            return (info1, _get_type_class(element), VarRef(t1, element))
        else:
            assert False

        
    elif isinstance(e, actlang.expr.SimpleAssignmentExpr):
        (info1,t1,vref1) = _analyze_expr(model, c, e.api_getLhsExpr())
        (info2,t2,vref2) = _analyze_expr(model, c, e.api_getRhsExpr())
        info1.union(info2)
        vref1.add_write_to_info(info1, c)
        vref2.add_read_to_info(info1, c)
        return (info1, t2, VarRef(None, None))


    elif isinstance(e, actlang.expr.CompoundAssignmentExpr):
        (info1,t1,vref1) = _analyze_expr(model, c, e.api_getLhsExpr())
        (info2,t2,vref2) = _analyze_expr(model, c, e.api_getRhsExpr())
        info1.union(info2)
        vref1.add_read_to_info(info1, c)
        vref1.add_write_to_info(info1, c)
        vref2.add_read_to_info(info1, c)
        return (info1, t2, VarRef(None, None))


    elif (isinstance(e, actlang.expr.CondOrExpr) or
          isinstance(e, actlang.expr.CondAndExpr)):
        (info1,t1,vref1) = _analyze_expr(model, c, e.api_getLeftOperand())
        (info2,t2,vref2) = _analyze_expr(model, c, e.api_getRightOperand())
        info1.union(info2)
        t = "primitive"
        vref1.add_read_to_info(info1, c)
        vref2.add_read_to_info(info1, c)
        return (info1, t, VarRef(None, None))


    elif (isinstance(e, actlang.expr.PrefixExpr) or
          isinstance(e, actlang.expr.PostfixExpr)):
        (info1,t1,vref1) = _analyze_expr(model, c, e.api_getOperand())
        op = e.api_getOperator()
        assert (isinstance(op, actlang.binaryop.Plus) or
                isinstance(op, actlang.binaryop.Minus))
        assert vref1.is_this_dot_var() or vref1.is_class_dot_var()
        vref1.add_read_to_info(info1, c)
        vref1.add_write_to_info(info1, c)
        return (info1, t1, VarRef(None, None))


    elif isinstance(e, actlang.expr.InfixExpr):
        (info1,t1,vref1) = _analyze_expr(model, c, e.api_getLeftOperand())
        (info2,t2,vref2) = _analyze_expr(model, c, e.api_getRightOperand())
        info1.union(info2)
        op = e.api_getOperator()
        if (isinstance(op, actlang.binaryop.Plus) or
            isinstance(op, actlang.binaryop.NumericalOperator) or
            isinstance(op, actlang.binaryop.NumericalBooleanOperator) or
            isinstance(op, actlang.binaryop.NumericalComparison) or
            isinstance(op, actlang.binaryop.EqualityOperator)):
            t = "primitive"
        else:
            assert False
        vref1.add_read_to_info(info1, c)
        vref2.add_read_to_info(info1, c)
        return (info1, t, VarRef(None, None))


    elif (isinstance(e, actlang.expr.UnaryMinusExpr) or
          isinstance(e, actlang.expr.LogicalNotExpr)):
        (info1,t1,vref1) = _analyze_expr(model, c, e.api_getOperand())
        vref1.add_read_to_info(info1, c)
        return (info1,"primitive", VarRef(None, None))


    elif (isinstance(e, actlang.expr.BoolLiteralExpr) or
          isinstance(e, actlang.expr.IntLiteralExpr) or
          isinstance(e, actlang.expr.StringLiteralExpr) or
          isinstance(e, actlang.expr.NullExpr)):
        return (Info(), "primitive", VarRef(None, None))


    elif isinstance(e, actlang.expr.ThisExpr):
        return (Info(), c, VarRef("this", None))


    elif isinstance(e, actlang.expr.InstanceCreationExpr):
        c_name = e.api_getTypeName()
        c = _find_class(model, c_name)
        assert c
        arg_list = e.api_getArgList()
        info = Info()
        for arg in arg_list:
            (arg_info,arg_t,arg_vref) = _analyze_expr(model, c, arg)
            arg_vref.add_read_to_info(arg_info, c)
            info.union(arg_info)
        info.objects_created.add(c)
        return (info, c, VarRef(None, None))


    assert False


def find_model_elements(element, element_type):
    elements = []

    if element.oclIsKindOf(element_type):
        elements.append(element)

    if element.oclIsKindOf(UML14.Namespace):
        for owned_element in element.ownedElement:
            elements.extend(find_model_elements(owned_element, element_type))
    if element.oclIsKindOf(UML14.StateMachine):
        for transition in element.transitions:
            elements.extend(find_model_elements(transition, element_type))
        elements.extend(find_model_elements(element.top, element_type))
    if element.oclIsKindOf(UML14.CompositeState):
        for subvertex in element.subvertex:
            elements.extend(find_model_elements(subvertex, element_type))

    return elements



if __name__ == '__main__':
    project = coral.loadDocument(sys.argv[1])
    models = [m for m in project.contents() if m.oclIsKindOf(UML14.Model)]
    if len(models) != 1:
        raise RuntimeError('document contains ' + str(len(models)) +
                           ' models instead of one')
    model = models[0]

    state_machines = find_model_elements(model, UML14.StateMachine)

    for sm in state_machines:
        c = sm.context
        if not c.oclIsKindOf(UML14.Class):
            print "A state machine not in context of a class"
            continue

        for t in sm.transitions:
            if t.effect:
                assert t.effect.script.body
                print "Analyzing statement '"+str(t.effect.script.body)+"' in class '"+str(c.name)+"'"
                info = analyze_statement(model, c, t.effect.script.body)
                print info
            if t.guard:
                assert t.guard.expression.body
                print "Analyzing expression '"+str(t.guard.expression.body)+"' in class '"+str(c.name)+"'"
                info = analyze_expression(model, c, t.guard.expression.body)
                print info
