# This file is part of the SMUML toolset.
# Copyright (C) 2007 Helsinki University of Technology.
#
# The SMUML toolset is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License version 2,
# as published by the Free Software Foundation.
#
# The SMUML toolset is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with the SMUML toolset.  If not, see <http://www.gnu.org/licenses/>.

"""Library of abstract types.

This package provides functionality for

  (via the module abstraction.typelibrary)

  i.   constructing abstract type libraries (represented by instances of
       the class abstraction.typelibrary.TypeLibrary) from one or more
       type specification files (see the help for abstaction.typelibrary
       for further information on the format of the type specification
       files),

  ii.  generating Python classes from the abstract types specified in
       the type library, and

  (via the module abstraction.typeassociation)

  iii. creating associations between attributes and type names by
       parsing type association files.

An example demonstrating the first two usage scenarios:

  # Import the abstraction package.
  import abstraction
  # Create an empty type library.
  lib = abstraction.typelibrary.TypeLibrary()
  # Initialize the library from one or more abstract type specification
  # files (give an iterable of file names as a parameter if there is more
  # than one file).  For convenience, file names that do not contain the
  # path separator are searched for, in addition to the current working
  # directory, in the colon-separated list of directories given in the
  # SMUML_ABSTRACT_TYPE_PATH environment variable.
  lib.read("typelibrary")
  # Check the basic semantic consistency of the type library (no
  # references to undefined types, well-definedness of operations and
  # tests defined for abstract types, no cycles in the "precision
  # hierarchy" of the types).
  lib.finalize()
  # Generate Python classes for the abstract types.  The classes will
  # be generated in the abstraction.typelibrary module.
  lib.generatePythonClasses()

For more information on the Python classes generated from the type
library, see the help for abstraction.typelibrary.

For more information on parsing type association files, see the help
for abstraction.typeassociation.

Author: Heikki Tauriainen <heikki.tauriainen@tkk.fi>
"""

__all__ = ["cartesize", "typehierarchy", "typelibrary", "typeassociation",
           "error"]

import typelibrary
import typeassociation
