# This file is part of the SMUML toolset.
# Copyright (C) 2007 Helsinki University of Technology.
#
# The SMUML toolset is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License version 2,
# as published by the Free Software Foundation.
#
# The SMUML toolset is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with the SMUML toolset.  If not, see <http://www.gnu.org/licenses/>.

"""Cartesian product of lists.

This module provides a function for enumerating over elements in the
Cartesian product of a given collection of domains represented as lists.

Author: Heikki Tauriainen <heikki.tauriainen@tkk.fi>
"""

def cartesize(*lists):
    """Generator for the Cartesian product of domains represented as lists.

    lists -- Lists representing the domains.
    """
    for l in lists:
        if len(l) == 0:
            raise StopIteration
    n = len(lists)
    e = n * [0]
    while True:
        yield [lists[i][j] for (i, j) in enumerate(e)]
        for i in xrange(n - 1, -1, -1):
            e[i] += 1
            if e[i] < len(lists[i]):
                break
            e[i] = 0
        else:
            raise StopIteration
