# This file is part of the SMUML toolset.
# Copyright (C) 2007 Helsinki University of Technology.
#
# The SMUML toolset is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License version 2,
# as published by the Free Software Foundation.
#
# The SMUML toolset is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with the SMUML toolset.  If not, see <http://www.gnu.org/licenses/>.

"""Error reporting utilities.

This module provides helper functions for reporting errors (and a base
class for exceptions raised in the abstractor code).

Author: Heikki Tauriainen <heikki.tauriainen@tkk.fi>
"""

import sys

# default stream for error messages
__stderr = sys.stderr

def getErrorStream():
    """Return the stream to which errors are printed."""
    global __stderr
    return __stderr

def setErrorStream(stream):
    """Change the stream to which errors are printed.

    Argument:
        stream -- A stream object.
    """
    global __stderr
    __stderr = stream

def abort(msg="an error occurred", exitcode=1):
    """Abort with an error message printed to the error stream.

    Arguments:
        msg      -- An error message.
        exitcode -- Exit code.
    """
    __stderr.write("%s:%s\n" % (sys.argv[0], msg))
    sys.exit(exitcode)

def warn(msg="warning"):
    """Print a warning to the error stream.

    Argument:
        msg -- A warning message.
    """
    __stderr.write("%s:warning:%s\n" % (sys.argv[0], msg))

def inform(msg=""):
    """Print an informational message the error stream.

    Argument:
        msg -- A message.
    """
    __stderr.write("%s:info:%s\n" % (sys.argv[0], msg))

class Error(Exception):
    """Exception with a string message (base class for all errors)."""

    def __init__(self, msg=""):
        """Initialize an Error with an optional message."""
        self.reason = msg

    def __str__(self):
        """Convert an Error object to a string."""
        return str(self.reason)

class OptionError(Error):
    """Exception for reporting command line option errors."""

    def __init__(self, msg=""):
        """Initialize an OptionError with an optional message."""
        Error.__init__(self, msg)

    
