#! /usr/bin/env python
# -*- coding: iso-8859-1 -*-

# This file is part of the SMUML toolset.
# Copyright (C) 2007 Helsinki University of Technology.
#
# The SMUML toolset is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License version 2,
# as published by the Free Software Foundation.
#
# The SMUML toolset is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with the SMUML toolset.  If not, see <http://www.gnu.org/licenses/>.

# interact.py
#
# 2005-09-01  Jori Dubrovin:  Added resetting after Ctrl-C
# 2005-08-08  Jori Dubrovin:  Using basetypes.py
# 2005-06-17  Jori Dubrovin:  Fixed vm initialization
# 2005-05-02  Jori Dubrovin:  Initial version
#

"""An interactive script for evaluating action language statements."""

import grammar
import basetypes
import errors


def interact():
    env = basetypes.getInitialEnvironment()
    vm = basetypes.getInitialMachine()
    print 'Type action language statements at the prompt below.'
    print 'Press Ctrl-C to return to the prompt.  Press Ctrl-D to exit.'
    while True:
        try:
            print
            input_ = raw_input('AL>>> ')
            try:
                line = 1
                while True:
                    try:
                        # parse may raise an IncompletenessError
                        # or a FatalStaticError.
                        tree = grammar.parse(input_)
                        # translate may raise an
                        # IncompletenessError or a
                        # FatalStaticError.  It is not allowed to
                        # mutate env in these cases.
                        main = tree.translate(env)
                        break
                    except errors.IncompletenessError:
                        # Ask for more input
                        line += 1
                        more = raw_input(str(line).rjust(2) + '... ')
                        if more:
                            input_ += '\n' + more
                            # Loop back to the line where input is
                            # parsed.
                        else:
                            # Reraise the IncompletenessError, which
                            # is catched as a StaticError below.
                            raise
                # Load the code into the machine.
                vm.initialize(main)
                # run may raise a RuntimeError_.
                vm.run()
            except errors.StaticError, e:
                print 'Static error:' + str(e)
            except errors.RuntimeError_, e:
                print 'Runtime error:\n' + str(e)
                env = basetypes.getInitialEnvironment()
                vm = basetypes.getInitialMachine()
        except KeyboardInterrupt:  # Catch Ctrl-C
            print '\nInterrupted'
            env = basetypes.getInitialEnvironment()
            vm = basetypes.getInitialMachine()
        except EOFError:  # Catch Ctrl-D
            print
            return


if __name__ == '__main__':
    interact()
