#!/usr/bin/python

#
# Prints in dotty format all state machines appearing in the model
#
# Author: Tommi Junttila
#

"""
Usage: print_state_machines_in_dotty.py inputmodel.xmi

Prints in dotty format all state machines appearing in the model in the
file inputfile.xmi.
"""

import sys
import coral
import Sim
import Sim.SMToDot
import Utils.Models

UML14 = coral.meta.UML14

if len(sys.argv) != 2:
    print __doc__
    sys.exit(1)

P = coral.loadDocument(sys.argv[1])

#
# Find the model
#
model = None
for m in P.contents():
    if m.oclIsKindOf(UML14.Model):
        if model:
            raise "The project contains two models - aborting!"
        model = m
if not model:
    raise "The project contains no model - aborting!"

state_machines = [sm for sm in model.__getAllParts__() \
                  if sm.oclIsKindOf(UML14.StateMachine)]

for (index,sm) in enumerate(state_machines):
    dot_filename = "debug_sm"+str(index)+".dot"
    print "Producing dotty file '"+dot_filename+"' for state machine '"+\
          Utils.Models.get_qualified_name(sm)+"'"
    f = open(dot_filename, 'w')
    Sim.SMToDot.SMToDot(f, sm)
    f.close()
