#! /usr/bin/env python
# -*- coding: iso-8859-1 -*-

# This file is part of the SMUML toolset.
# Copyright (C) 2007 Helsinki University of Technology.
#
# The SMUML toolset is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License version 2,
# as published by the Free Software Foundation.
#
# The SMUML toolset is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with the SMUML toolset.  If not, see <http://www.gnu.org/licenses/>.

# suboco.py
#
# 2007-12-20  Jori Dubrovin:  Suboco 1.10
# 2007-12-16  Jori Dubrovin:  Suboco 1.09
# 2007-11-22  Jori Dubrovin:  Suboco 1.08
# 2007-11-21  Jori Dubrovin:  Suboco 1.07
# 2007-11-21  Jori Dubrovin:  Suboco 1.06
# 2007-11-20  Jori Dubrovin:  Suboco 1.05
# 2007-10-02  Jori Dubrovin:  Suboco 1.04
# 2007-09-05  Jori Dubrovin:  Suboco 1.03
# 2007-09-01  Jori Dubrovin:  Suboco 1.02
# 2007-08-30  Jori Dubrovin:  Suboco 1.01
# 2007-08-27  Jori Dubrovin:  Suboco 1.00
#

"""UML bounded model checker."""

__version__ = '1.10'

import os
import sys
import exceptions

import PySMT
from Suboco import errors
from Suboco import models
from Suboco.coralutils import uml, coral
from Suboco import options


def encodeXmi(inFilename):
    project = coral.loadDocument(inFilename)
    umlModels = [e for e in project.contents() if e.oclIsKindOf(uml.Model)]
    if len(umlModels) != 1:
        raise errors.InputError('document contains ' + str(len(umlModels)) +
                                ' models instead of one')
    m = models.Model(umlModels[0])
    if options.opt.dot_file:
        dotFile = open(options.opt.dot_file, 'w')
        for line in options.comment:
            dotFile.write('//' + line + '\n')
        m.getCircuit().to_dot(dotFile)
        dotFile.close()
    return m

def getSolver():
    """Return the chosen PySMT.Solver."""
    if options.opt.z3:
        if options.opt.format != 'smtlib':
            raise errors.NotSupportedError('Native format for Z3 is not '
                                           'supported.  Use --format=smtlib.')
        return PySMT.Z3(options.opt.z3)
    if options.opt.mathsat:
        return PySMT.MathSAT(options.opt.mathsat)
    if options.opt.cvc3:
        return PySMT.CVC3(options.opt.cvc3)
    if options.opt.stp:
        return PySMT.STP(options.opt.stp)
    return PySMT.Yices(options.opt.yices)

def doBmc(inFilename):
    model = encodeXmi(inFilename)
    solver = getSolver()
    bound = options.opt.min_bound
    try:
        while bound <= options.opt.max_bound:
            sys.stdout.write('Bound %2d, ' % bound)
            sys.stdout.flush()
            problem = model.unroll(bound)
            sys.stdout.write('gates %5d ' % problem.get_nof_gates())
            sys.stdout.flush()
            sys.stdout.write('(relevant %5d)  ' %
                             problem.get_nof_relevant_gates())
            sys.stdout.flush()
            # To print the problem:
            # print '\n'.join([str(tuple([str(i) for
            #                             i in g.get_structural_id()])) for
            #                  g in problem._gates])
            result = problem.solve(solver, options.opt.format)
            print result
            if result == 'sat':
                traceFilename = options.opt.trace_file
                if not traceFilename:
                    traceFilename = os.path.splitext(inFilename)[0] + '.trace'
                traceFile = open(traceFilename, 'w')
                traceFile.write(model.extractTrace(inFilename, problem))
                traceFile.close()
                print 'Trace written to ' + traceFilename,
                break
            if result != 'unsat':
                raise exceptions.RuntimeError('PySMT solve returned ' +
                                              str(result))
            bound += 1
    finally:
        print
        if hasattr(os, 'times'):
            # os.times()[2] is the total user time spent in child
            # processes.  os.times()[3] is the corresponding system time.
            times = os.times()
            print 'Total solver CPU time %.3f s' % (times[2] + times[3])

if __name__ == '__main__':
    inFilename = options.initializeOptions(__version__)
    doBmc(inFilename)
