#! /usr/bin/env python
# -*- coding: iso-8859-1 -*-

# This file is part of the SMUML toolset.
# Copyright (C) 2007 Helsinki University of Technology.
#
# The SMUML toolset is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License version 2,
# as published by the Free Software Foundation.
#
# The SMUML toolset is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with the SMUML toolset.  If not, see <http://www.gnu.org/licenses/>.

# uboco.py
#
# 2007-10-02  Jori Dubrovin:  Version 1.10
# 2007-09-21  Jori Dubrovin:  Version 1.09
# 2007-09-05  Jori Dubrovin:  Version 1.08
# 2007-07-17  Jori Dubrovin:  Version 1.07
# 2007-06-20  Jori Dubrovin:  Version 1.06
# 2007-05-07  Jori Dubrovin:  Version 1.05
# 2007-05-04  Jori Dubrovin:  Version 1.04
# 2007-04-04  Jori Dubrovin:  Version 1.03
# 2007-03-08  Jori Dubrovin:  Version 1.02
# 2007-03-02  Jori Dubrovin:  Version 1.01
# 2007-02-05  Jori Dubrovin:  Version 1.00
# 2006-12-04  Jori Dubrovin:  Using optparse for options
# 2006-08-22  Jori Dubrovin:  Fixed syntax bugs
# 2006-08-15  Jori Dubrovin:  Added deadlock specs
# 2006-07-30  Jori Dubrovin:  Initial version
#

"""Translation of UML to NuSMV."""

__version__ = '1.10'

import sys
import time

from BMC import errors
from BMC import models
from BMC.coralutils import uml, coral
from BMC import options


def translateXmiToSmv(inFilename, smvFilename):
    project = coral.loadDocument(inFilename)
    umlModels = [e for e in project.contents() if e.oclIsKindOf(uml.Model)]
    if len(umlModels) != 1:
        raise errors.InputError('document contains ' + str(len(umlModels)) +
                                ' models instead of one')
    m = models.Model(umlModels[0])
    c = ['-- NuSMV input file generated by Uboco ' + __version__,
         '-- Input file: ' + inFilename,
         '-- Created on ' + time.ctime(time.time())]
    c += ['-- Options:']
    optionNames = options.opt.__dict__.keys()
    optionNames.sort()
    for n in optionNames:
        c += ['--   ' + n + ': ' + str(options.opt.__dict__[n])]
    c += m.elementMap(inFilename)
    c += m.code()
    smvFile = open(smvFilename, 'w')
    smvFile.write('\n'.join(c + ['']))
    smvFile.close()


if __name__ == '__main__':
    (inFileName, smvFileName) = options.initializeOptions(__version__)
    translateXmiToSmv(inFileName, smvFileName)
